﻿using UnityEngine;
using System.Collections;

public class gjCamera : MonoBehaviour
{
    // Serializabled fields

    [Header("X")]

    [SerializeField]
    private float m_MinX = 0f;
    [SerializeField]
    private float m_MaxX = 0f;

    [SerializeField]
    private float m_XMovementThreshold = 0f;

    [SerializeField]
    private float m_XSpeed = 5f;

    [Header("Y")]

    [SerializeField]
    private float m_MinY = 0f;
    [SerializeField]
    private float m_MaxY = 0f;

    [SerializeField]
    private float m_YMovementThreshold = 0f;

    [SerializeField]
    private float m_YSpeed = 5f;

    [Header("Logic")]

    [SerializeField]
    private Transform m_Target = null;

    // Fields

    private Vector3 m_NextPosition = Vector3.zero;

    // ACCESSORS

    public float minX
    {
        get
        {
            return m_MinX;
        }
    }

    public float maxX
    {
        get
        {
            return Mathf.Max(minX, m_MaxX);
        }
    }

    public float xMovementThreshold
    {
        get
        {
            return m_XMovementThreshold;
        }
    }

    public float xSpeed
    {
        get
        {
            return Mathf.Max(0f, m_XSpeed);
        }
    }

    public float minY
    {
        get
        {
            return m_MinY;
        }
    }

    public float maxY
    {
        get
        {
            return Mathf.Max(minY, m_MaxY);
        }
    }

    public float yMovementThreshold
    {
        get
        {
            return m_YMovementThreshold;
        }
    }

    public float ySpeed
    {
        get
        {
            return Mathf.Max(0f, m_YSpeed);
        }
    }

    // MonoBehaviour's interface

    void Start()
    {
        if (m_Target != null)
        {
            Vector3 nextPosition = m_Target.transform.position;

            nextPosition.x = Mathf.Clamp(nextPosition.x, minX, maxX);
            nextPosition.y = Mathf.Clamp(nextPosition.y, minY, maxY);


            nextPosition = new Vector3(nextPosition.x, nextPosition.y, m_Target.transform.position.z);
            transform.position = new Vector3(nextPosition.x, nextPosition.y, transform.position.z);
            m_NextPosition = nextPosition;
        }
    }


    void FixedUpdate()
    {
        if (m_Target == null)
            return;

        Vector2 myPosition = transform.position;
        Vector2 targetPosition = m_Target.position;

        Vector2 offset = targetPosition - myPosition;

        float offsetX = offset.x;
        float offsetY = offset.y;

        float absOffsetX = Mathf.Abs(offsetX);
        float absOffsetY = Mathf.Abs(offsetY);

        bool moveX = (absOffsetX > xMovementThreshold);
        bool moveY = (absOffsetY > yMovementThreshold);

        bool move = (moveX || moveY);

        float movementX = 0f;
        float movementY = 0f;

        if (moveX)
        {
            float direction = (offsetX > 0f) ? 1f : -1f;
            movementX = direction * xSpeed * Time.fixedDeltaTime;
        }

        if (moveY)
        {
            float direction = (offsetY > 0f) ? 1f : -1f;
            movementY = direction * ySpeed * Time.fixedDeltaTime;
        }

        if (move)
        {
            Vector3 cameraMovement = new Vector3(movementX, movementY, 0f);
            Vector3 nextPosition = transform.position + cameraMovement;

            nextPosition.x = Mathf.Clamp(nextPosition.x, minX, maxX);
            nextPosition.y = Mathf.Clamp(nextPosition.y, minY, maxY);

            m_NextPosition = nextPosition;
        }

        Vector2 targetNewPos = Vector2.Lerp(myPosition, m_NextPosition, 0.9f);
        transform.position = new Vector3(targetNewPos.x, targetNewPos.y, transform.position.z);
    }

    // LOGIC

    public void SetTransform(Transform i_Transform)
    {
        m_Target = i_Transform;

        if (m_Target != null)
        {
            Vector3 nextPosition = m_Target.transform.position;

            nextPosition.x = Mathf.Clamp(nextPosition.x, minX, maxX);
            nextPosition.y = Mathf.Clamp(nextPosition.y, minY, maxY);

            nextPosition = new Vector3(nextPosition.x, nextPosition.y, m_Target.transform.position.z);
            transform.position = new Vector3(nextPosition.x, nextPosition.y, transform.position.z);
            m_NextPosition = nextPosition;
        }
    }
}
