﻿using UnityEngine;

public class gjCharacterInput : MonoBehaviour
{
    // Serializable fields

    [SerializeField]
    private bool m_InputEnabled = true;
    [SerializeField]
    private bool m_DisableInputOnWave = false;

    // Fields

    private bool m_InputAllowed = true;

    // MonoBehaviour's interface

    void OnEnable()
    {
        Messenger.AddListener("WaveStarted", OnWaveStarted);
        Messenger.AddListener("WaveEnded", OnWaveEnded);
    }

    void OnDisable()
    {
        Messenger.RemoveListener("WaveStarted", OnWaveStarted);
        Messenger.RemoveListener("WaveEnded", OnWaveEnded);
    }

    // ACCESSORS

    public bool inputEnabled
    {
        get
        {
            return m_InputEnabled;
        }
    }

    public bool inputAllowed
    {
        get
        {
            return m_InputAllowed;
        }
    }

    public bool inputActive
    {
        get
        {
            return inputEnabled && inputAllowed;
        }
    }

    // LOGIC

    public void SetEnabled(bool i_Enabled)
    {
        m_InputEnabled = i_Enabled;
    }

    public void SetAllowed(bool i_Allowed)
    {
        m_InputAllowed = i_Allowed;
    }

    public float GetAxis(string i_ActionId)
    {
        if (inputActive)
        {
            return Input.GetAxis(i_ActionId);
        }

        return 0f;
    }

    public bool GetButton(string i_ActionId)
    {
        if (inputActive)
        {
            return Input.GetButton(i_ActionId);
        }

        return false;
    }

    public bool GetButtonDown(string i_ActionId)
    {
        if (inputActive)
        {
            return Input.GetButtonDown(i_ActionId);
        }

        return false;
    }

    public bool GetButtonUp(string i_ActionId)
    {
        if (inputActive)
        {
            return Input.GetButtonUp(i_ActionId);
        }

        return false;
    }

    public bool GetKey(KeyCode i_KeyCode)
    {
        if (inputActive)
        {
            return Input.GetKey(i_KeyCode);
        }

        return false;
    }

    public bool GetKeyDown(KeyCode i_KeyCode)
    {
        if (inputActive)
        {
            return Input.GetKeyDown(i_KeyCode);
        }

        return false;
    }

    public bool GetKeyUp(KeyCode i_KeyCode)
    {
        if (inputActive)
        {
            return Input.GetKeyUp(i_KeyCode);
        }

        return false;
    }

    // EVENTS

    private void OnWaveStarted()
    {
        if (m_DisableInputOnWave)
        {
            SetEnabled(false);
        }
    }

    private void OnWaveEnded()
    {
        if (m_DisableInputOnWave)
        {
            SetEnabled(true);
        }
    }
}
