﻿using UnityEngine;
using UnityEngine.UI;
using UnityEngine.SceneManagement;

using System;
using System.Collections;

public class gjIntro : MonoBehaviour
{
    // STATIC

    private static string s_NextScene = "Wait";

    // Serializable fields

    [SerializeField]
    private Image m_Logo = null;

    [SerializeField]
    private float m_FadeInTime = 0f;
    [SerializeField]
    private float m_Duration = 0f;
    [SerializeField]
    private float m_FadeOutTime = 0f;

    // MonoBehaviour's interface

    void Start()
    {
        StartCoroutine(Sequence(OnSequenceCompleted));
    }

    // INTERNALS

    private IEnumerator Sequence(Action i_CompletedCallback = null)
    {
        // Fade in.

        {
            float duration = m_FadeInTime;
            float time = 0f;

            while (time < duration)
            {
                float t = Mathf.Clamp01(time / duration);
                SetImageAlpha(m_Logo, t);

                yield return null;
                time += Time.deltaTime;
            }
        }

        // Rest

        {
            float duration = m_Duration;
            yield return new WaitForSeconds(duration);
        }

        // Fade out.

        {
            float duration = m_FadeOutTime;
            float time = 0f;

            while (time < duration)
            {
                float t = Mathf.Clamp01(time / duration);
                SetImageAlpha(m_Logo, 1f - t);

                yield return null;
                time += Time.deltaTime;
            }
        }

        if (i_CompletedCallback != null)
        {
            i_CompletedCallback();
        }
    }

    private void OnSequenceCompleted()
    {
        LoadNextScene();
    }

    private void LoadNextScene()
    {
        SceneManager.LoadScene(s_NextScene, LoadSceneMode.Single);
    }

    // UTILS

    private void SetImageSprite(Image i_Image, Sprite i_Sprite)
    {
        if (i_Image != null)
        {
            i_Image.sprite = i_Sprite;
        }
    }

    private void SetImageAlpha(Image i_Image, float i_Alpha)
    {
        if (i_Image != null)
        {
            Color currentColor = i_Image.color;
            currentColor.a = i_Alpha;
            i_Image.color = currentColor;
        }
    }
}
