﻿using UnityEngine;

public class gjMask : MonoBehaviour
{
    // Serializable fields

    [SerializeField]
    private Sprite m_CircleSprite = null;

    [SerializeField]
    private SpriteRenderer m_White = null;
    [SerializeField]
    private SpriteRenderer m_Black = null;

    // ACCESSORS

    public float whiteRay
    {
        get
        {
            if (m_White == null || m_CircleSprite == null)
            {
                return 0f;
            }

            Transform whiteTransform = m_White.transform;
            int spriteRay = m_CircleSprite.texture.height;
            float ray = (float)spriteRay * whiteTransform.localScale.x;

            return ray / 100f; // 100f is pixel for unit.
        }

        set
        {
            if (m_White == null || m_CircleSprite == null)
                return;

            float ray = value * 100f; // 100f is pixel for unit.
            int spriteRay = m_CircleSprite.texture.height;
            float localScale = ray / (float)spriteRay;

            Transform whiteTransform = m_White.transform;
            Vector3 targetLocalScale = new Vector3(localScale, localScale, localScale);
            whiteTransform.localScale = targetLocalScale;
        }
    }

    public float blackRay
    {
        get
        {
            if (m_Black == null || m_CircleSprite == null)
            {
                return 0f;
            }

            Transform blackTransform = m_Black.transform;
            int spriteRay = m_CircleSprite.texture.height;
            float ray = (float)spriteRay * blackTransform.localScale.x;

            return ray / 100f; // 100f is pixel for unit.
        }

        set
        {
            if (m_Black == null || m_CircleSprite == null)
                return;

            float ray = value * 100f; // 100f is pixel for unit.
            int spriteRay = m_CircleSprite.texture.height;
            float localScale = ray / (float)spriteRay;

            Transform blackTransform = m_Black.transform;
            Vector3 targetLocalScale = new Vector3(localScale, localScale, localScale);
            blackTransform.localScale = targetLocalScale;
        }
    }

    // MonoBehaviour's interface

    void Awake()
    {
        if (m_White != null)
        {
            m_White.sprite = m_CircleSprite;
            m_White.color = Color.white;
        }

        if (m_Black != null)
        {
            m_Black.sprite = m_CircleSprite;
            m_Black.color = Color.black;
        }
    }

    // LOGIC

    public void SetCenter(Vector2 i_Center)
    {
        Vector3 targetPosition = new Vector3(i_Center.x, i_Center.y, transform.position.z);
        transform.position = targetPosition;
    }

    public void SetAmplitudeAndSize(float i_Amplitude, float i_Size)
    {
        whiteRay = Mathf.Max(0f, i_Amplitude);
        blackRay = Mathf.Max(0f, whiteRay - i_Size);
    }

    public void SetRays(float i_White, float i_Black)
    {
        whiteRay = Mathf.Max(0f, i_White);
        blackRay = Mathf.Max(0f, i_Black);
    }

    public void Clear()
    {
        SetRays(0f, 0f);
        SetAlpha(1f);
        SetCenter(Vector2.zero);
    }

    public void SetAlpha(float i_Alpha)
    {
        if (m_White == null)
            return;

        float alpha = Mathf.Clamp01(i_Alpha);
        Color currentColor = m_White.color;
        currentColor.a = alpha;
        m_White.color = currentColor;
    }
}
