﻿using UnityEngine;

[RequireComponent(typeof(Camera))]
public class gjMaskCamera : MonoBehaviour
{
    // Serializable fields

    [Header("Mask")]

    [SerializeField]
    private Shader m_MaskShader = null;
    [SerializeField]
    private gjMask m_MaskPrefab = null;

    // Fields

    private Camera m_Camera = null;
    private Camera m_MaskCamera = null;

    private gjMask m_Mask = null;

    private Material m_MaskShaderMaterial = null;

    private bool m_Initialized = false;

    private RenderTexture m_RenderTexture = null;

    // ACCESSORS

    public bool initialized
    {
        get
        {
            return m_Initialized;
        }
    }

    public gjMask mask
    {
        get
        {
            return m_Mask;
        }
    }

    // MonoBehaviour's interface

    void Awake()
    {
        m_Camera = GetComponent<Camera>();
        CreateMaterials();
        CreateMaskCamera();
        m_Initialized = true;
    }

    void OnDestroy()
    {
        m_Initialized = false;

        if (m_MaskCamera != null)
        {
            Destroy(m_MaskCamera.gameObject);
            m_MaskCamera = null;
        }

        DestroyMaterials();
    }

    // Render pipeline callbakcs
    
    void OnPreCull()
    {
        // Create render texture.

        int width = m_Camera.pixelWidth;
        int height = m_Camera.pixelHeight;
        m_RenderTexture = RenderTexture.GetTemporary(width, height, 16, RenderTextureFormat.Default);

        // Set mask camera target texture.

        m_MaskCamera.targetTexture = m_RenderTexture;

        // Render to texture.

        m_MaskCamera.Render();
    }

    void OnRenderImage(RenderTexture i_Source, RenderTexture i_Destination)
    {
        m_MaskShaderMaterial.SetTexture("_MaskSource", m_RenderTexture);
        Graphics.Blit(i_Source, i_Destination, m_MaskShaderMaterial);
        RenderTexture.ReleaseTemporary(m_RenderTexture);
    }

    // INTERNALS

    private void CreateMaterials()
    {
        if (m_MaskShaderMaterial == null)
        {
            m_MaskShaderMaterial = new Material(m_MaskShader);
            m_MaskShaderMaterial.hideFlags = HideFlags.DontSave;
        }
    }

    private void DestroyMaterials()
    {
        Destroy(m_MaskShaderMaterial);
        m_MaskShaderMaterial = null;
    }

    private void CreateMaskCamera()
    {
        GameObject maskCameraGo = new GameObject("MaskCamera");
        maskCameraGo.hideFlags = HideFlags.DontSave;
        maskCameraGo.transform.SetParent(transform, true);
        m_MaskCamera = maskCameraGo.AddComponent<Camera>();

        m_MaskCamera.CopyFrom(m_Camera);
        m_MaskCamera.renderingPath = RenderingPath.Forward;
        m_MaskCamera.enabled = false;
        m_MaskCamera.backgroundColor = new Color(0f, 0f, 0f, 0f);
        m_MaskCamera.clearFlags = CameraClearFlags.SolidColor;
        m_MaskCamera.cullingMask = LayerMask.GetMask("Mask");

        if (m_MaskPrefab != null)
        {
            gjMask mask = Instantiate<gjMask>(m_MaskPrefab);
            mask.transform.SetParent(maskCameraGo.transform, false);
            mask.transform.localPosition = new Vector3(0f, 0f, 2f);
            mask.transform.localRotation = Quaternion.identity;

            m_Mask = mask;
        }
    }
}
