﻿using UnityEngine;
using UnityEngine.UI;
using UnityEngine.SceneManagement;

public class gjMenuManager : MonoBehaviour
{
    // STATIC

    private static string s_SequenceScene = "StorySequence";
    private static string s_GameScene = "Game";

    // Serializable fields

    [Header("Logic")]

    [SerializeField]
    private bool m_SkipSequence = false;

    [Header("Widgets")]

    [SerializeField]
    private Button m_PlayButton = null;
    [SerializeField]
    private Button m_QuitButton = null;

    [Header("Audio")]

    [SerializeField]
    private AudioClip m_BackgroundMusic = null;

    // Fields

    private bool m_PlayRequested = false;
    private bool m_QuitRequested = false;

    // Components

    private AudioSource m_AudioSource = null;

    // MonoBehaviour's interface

    void Awake()
    {
        m_AudioSource = GetComponent<AudioSource>();
        if (m_AudioSource == null)
        {
            m_AudioSource = gameObject.AddComponent<AudioSource>();
        }

        m_AudioSource.clip = m_BackgroundMusic;
        m_AudioSource.loop = true;
        m_AudioSource.playOnAwake = false;
    }

    void OnEnable()
    {
        if (m_PlayButton != null)
        {
            m_PlayButton.onClick.AddListener(OnPlayButtonClicked);
        }

        if (m_QuitButton != null)
        {
            m_QuitButton.onClick.AddListener(OnQuitButtonClicked);
        }
    }

    void OnDisable()
    {
        if (m_PlayButton != null)
        {
            m_PlayButton.onClick.RemoveListener(OnPlayButtonClicked);
        }

        if (m_QuitButton != null)
        {
            m_QuitButton.onClick.RemoveListener(OnQuitButtonClicked);
        }
    }

    void Start()
    {
        m_AudioSource.Play();
    }

    // EVENTS

    private void OnPlayButtonClicked()
    {
        if (m_PlayRequested || m_QuitRequested)
            return;

        m_PlayRequested = true;

        m_AudioSource.Stop();

        if (m_SkipSequence)
        {
            SceneManager.LoadScene(s_GameScene, LoadSceneMode.Single);
        }
        else
        {
            SceneManager.LoadScene(s_SequenceScene, LoadSceneMode.Single);
        }
    }

    private void OnQuitButtonClicked()
    {
        if (m_PlayRequested || m_QuitRequested)
            return;

        m_QuitRequested = true;

        m_AudioSource.Stop();

        Application.Quit();
    }
}