﻿using UnityEngine;

public class gjWaveController : MonoBehaviour
{
    [Header("Wave")]

    [SerializeField]
    private float m_WaveDuration = 2f;
    [SerializeField]
    private float m_StartFadeAt = 1f;
    [SerializeField]
    private float m_WaveSize = 1f;
    [SerializeField]
    private float m_WaveGrowRatio = 1f;

    [Header("Debug")]

    [SerializeField]
    private bool m_Debug = false;

    // Fields

    private gjMask m_Mask = null;

    private float m_CurrentAmplitude = 0f;
    private float m_CurrentTime = 0f;

    private bool m_WaveRunning = false;

    // ACCESSORS

    public bool waveRunning
    {
        get { return m_WaveRunning; }
    }

    public float currentMinAmplitude
    {
        get { return Mathf.Max(0f, m_CurrentAmplitude - m_WaveSize); }
    }

    public float currentMaxAmplitude
    {
        get { return m_CurrentAmplitude; }
    }

    public float currentSize
    {
        get { return (currentMaxAmplitude - currentMinAmplitude); }
    }

    public float currentTime
    {
        get { return m_CurrentTime; }
    }

    public float startFadeTime
    {
        get
        {
            float startTime = Mathf.Clamp(m_StartFadeAt, 0f, m_WaveDuration);
            return startTime;
        }
    }

    public float startFadePercentage
    {
        get
        {
            return Mathf.Clamp01(startFadeTime / m_WaveDuration);
        }
    }

    public float timePercentage
    {
        get { return Mathf.Clamp01(m_CurrentTime / m_WaveDuration); }
    }

    public gjMask mask
    {
        get
        {
            if (m_Mask == null)
            {
                Camera mainCamera = Camera.main;
                gjMaskCamera maskCamera = mainCamera.GetComponent<gjMaskCamera>();
                if (maskCamera != null)
                {
                    m_Mask = maskCamera.mask;
                }
            }

            return m_Mask;
        }
    }

    // MonoBehaviour's interface

    private void Update()
    {
        // Update wave.

        if (m_WaveRunning)
        {
            // Update.

            float deltaTime = Time.deltaTime;
            m_CurrentTime += deltaTime;
            if (m_CurrentTime > m_WaveDuration)
            {
                StopWave();
            }
            else
            {
                m_CurrentAmplitude += m_WaveGrowRatio * deltaTime;
            }

            // Set wave data.

            if (mask != null)
            {
                mask.SetAmplitudeAndSize(currentMaxAmplitude, currentSize);
                if (timePercentage > startFadePercentage)
                {
                    float v = (currentTime - startFadeTime) / (m_WaveDuration - startFadeTime);
                    mask.SetAlpha(1f - v);
                }
            }
        }
    }

    // LOGIC

    public void StartWave(Vector2 i_Center)
    {
        if (m_WaveRunning)
            return;

        m_WaveRunning = true;

        m_CurrentAmplitude = 0f;
        m_CurrentTime = 0;

        if (mask != null)
        {
            mask.SetCenter(i_Center);
        }

        Messenger.Broadcast("WaveStarted");

        if (m_Debug)
        {
            Debug.Log("Wave Started");
        }
    }

    public void StopWave()
    {
        if (!m_WaveRunning)
            return;

        m_WaveRunning = false;

        m_CurrentAmplitude = 0f;
        m_CurrentTime = 0f;

        if (mask != null)
        {
            mask.Clear();
        }

        Messenger.Broadcast("WaveEnded");

        if (m_Debug)
        {
            Debug.Log("Wave Ended");
        }
    }
}