﻿Shader "Hidden/MaskShader"
{
	Properties
	{
		_MainTex("Base (RGB)", 2D) = "white" {}
	}
	
	SubShader
	{
		Tags
		{
			"RenderType" = "Opaque"
		}

		Pass
		{
			Lighting Off

			ZTest Always
			ZWrite Off
			Cull Off

			CGPROGRAM

			#pragma vertex vert
			#pragma fragment frag
			#pragma target 3.0

			#include "UnityCG.cginc"

			sampler2D _MainTex;
			sampler2D _MaskSource;

			struct v2f
			{
				float4 pos : SV_POSITION;
				float2 uv : TEXCOORD0;
			};

			v2f vert(appdata_img v)
			{
				v2f o;
				o.pos = mul(UNITY_MATRIX_MVP, v.vertex);
				o.uv = v.texcoord;
				return o;
			}

			fixed4 frag(v2f input) : COLOR
			{
				float2 uv = input.uv;
				float2 uv2 = float2(uv.x, 1 - uv.y);

				fixed4 originalPixel = tex2D(_MainTex, uv);
				fixed4 maskSource = tex2D(_MaskSource, uv2);

				float maskThreshold = 0.01;

				if (maskSource.r > maskThreshold && maskSource.g > maskThreshold && maskSource.b > maskThreshold)
				{
					float alpha = originalPixel.a;
					originalPixel *= maskSource;
					originalPixel.a = alpha;
					return originalPixel;
				}

				return fixed4(0, 0, 0, 1);
			}
			
			ENDCG
		}	
	}
			
	Fallback "Unlit/Transparent" 
}

